@@ -1,19 +1,18 @@
-;;;; texinfo.el
+;;; texinfo.el--major mode for editing Texinfo files.
 
-;;; Major mode for editing Texinfo files.
+;; Copyright (C) 1985, '88, '89, '90, '91,
+;;                '92, '93, '96 Free Software Foundation, Inc.
 
-;;; Version 2.13   18 March 1993
-;;; Robert J. Chassell          
+;; Author: Robert J. Chassell          
+;; Date:   6 Sep 1996
+;; Maintainer: bug-texinfo@prep.ai.mit.edu
+;; Keywords: maint, tex, docs
 
-;; Copyright (C) 1985, '88, '89, 
-;;                '90, '91, '92, '93 Free Software Foundation, Inc.
-
-
-;;; This file is part of GNU Emacs.
+;; This file is part of GNU Emacs.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 1, or (at your option)
+;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
@@ -22,134 +21,44 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 
-;;; Autoloads
-
-(autoload 'texinfo-format-region
-          "texinfmt"
-  "Convert the current region of the Texinfo file to Info format.
-This lets you see what that part of the file will look like in Info.
-The command is bound to \\[texinfo-format-region].  The text that is
-converted to Info is stored in a temporary buffer."
-          t nil)
-
-(autoload 'texinfo-format-buffer
-          "texinfmt"
-  "Process the current buffer as texinfo code, into an Info file.
-The Info file output is generated in a buffer visiting the Info file
-names specified in the @setfilename command.
-
-Non-nil argument (prefix, if interactive) means don't make tag table
-and don't split the file if large.  You can use Info-tagify and
-Info-split to do these manually."
-          t nil)
+;;; Autoloads:
 
 (autoload 'makeinfo-region
-          "makeinfo"
-  "Make Info file from region of current Texinfo source file.
+  "makeinfo"
+  "Make Info file from region of current Texinfo file, and switch to it.
 
-The \\[next-error] command can be used to move to the next error \(if
-any are found\)."
-          t nil)
+This command does not offer the `next-error' feature since it would
+apply to a temporary file, not the original; use the `makeinfo-buffer'
+command to gain use of `next-error'."
+  t nil)
 
 (autoload 'makeinfo-buffer
-          "makeinfo"
+  "makeinfo"
   "Make Info file from current buffer.
 
-The \\[next-error] command can be used to move to the next error 
-\(if any are found\)."
-          t nil)
+Use the \\[next-error] command to move to the next error 
+\(if there are errors\)."
+  t nil)
 
 (autoload 'kill-compilation
-          "compile"
+  "compile"
   "Kill the process made by the \\[compile] command."
-          t nil)
+  t nil)
 
 (autoload 'makeinfo-recenter-compilation-buffer
-          "makeinfo"
-  "Redisplay *compilation* buffer so that most recent output can be seen.
+  "makeinfo"
+  "Redisplay `*compilation*' buffer so most recent output can be seen.
 The last line of the buffer is displayed on
 line LINE of the window, or centered if LINE is nil."
-          t nil)
-
-(autoload 'texinfo-tex-region
-          "texnfo-tex"
-  "Run tex on the current region. 
-
-A temporary file is written in the default directory, and tex is run
-in that directory.  The first line of the file is copied to the
-temporary file; and if the buffer has a header, it is written to the
-temporary file before the region itself.  The buffer's header is all
-lines between the strings defined by texinfo-start-of-header and
-texinfo-end-of-header inclusive.  The header must start in the first 100
-lines.  The value of texinfo-tex-trailer is appended to the temporary file
-after the region."
-          t nil)
-
-(autoload 'texinfo-tex-buffer
-          "texnfo-tex"
-  "Run TeX on current buffer.
-After running TeX the first time, you may have to run \\[texinfo-texindex]
-and then \\[texinfo-tex-buffer] again."
-          t nil)
-
-(autoload 'texinfo-kill-tex-job
-          "texnfo-tex"
-  "Kill the currently running TeX job."
-          t nil)
-
-(autoload 'texinfo-recenter-tex-output-buffer
-          "texnfo-tex"
-  "Redisplay buffer of TeX job output so that most recent output can be seen.
-The last line of the buffer is displayed on
-line LINE of the window, or centered if LINE is nil."
-          t nil)
-
-(autoload 'texinfo-delete-from-tex-print-queue
-          "texnfo-tex"
-  "Delete job from the line printer spooling queue.
-You are prompted for the job number (shown by a previous
-\\[texinfo-show-tex-print-queue] command."
-          t nil)
-
-(autoload 'texinfo-show-tex-print-queue
-          "texnfo-tex"
-  "Show the print queue that \\[texinfo-tex-print] put your job on.
-Runs the shell command defined by texinfo-show-tex-queue-command."
-          t nil)
-
-(autoload 'texinfo-tex-print
-          "texnfo-tex"
-  "Print .dvi file made by \\[texinfo-tex-region] or \\[texinfo-tex-buffer].
-Runs the shell command defined by texinfo-tex-dvi-print-command."
-          t nil)
-
-(autoload 'texinfo-texindex
-          "texnfo-tex"
-  "Run texindex on unsorted index files.
-The index files are made by \\[texinfo-tex-region] or \\[texinfo-tex-buffer].
-Runs the shell command defined by texinfo-texindex-command."
-          t nil)
-
-(autoload 'texinfo-make-menu
-          "texnfo-upd"
-  "Without any prefix argument, make or update a menu.
-Make the menu for the section enclosing the node found following point.
-
-Non-nil argument (prefix, if interactive) means make or update menus
-for nodes within or part of the marked region.
-
-Whenever a menu exists, and is being updated, the descriptions that
-are associated with node names in the pre-existing menu are
-incorporated into the new menu.  Otherwise, the nodes' section titles
-are inserted as descriptions."
-          t nil)
+  t nil)
 
 (autoload 'texinfo-update-node
-          "texnfo-upd"
+  "texnfo-upd"
   "Without any prefix argument, update the node in which point is located.
 Non-nil argument (prefix, if interactive) means update the nodes in the
 marked region.
@@ -168,25 +77,58 @@ keybindings, are:
     texinfo-indent-menu-description (column &optional region-p)
 
 The `texinfo-column-for-description' variable specifies the column to
-which menu descriptions are indented. Its default value is 24."
-          t nil)
+which menu descriptions are indented. Its default value is 32."
+  t nil)
 
 (autoload 'texinfo-every-node-update
-          "texnfo-upd"
+  "texnfo-upd"
   "Update every node in a Texinfo file."
-          t nil)
+  t nil)
+
+(autoload 'texinfo-sequential-node-update
+  "texnfo-upd"
+  "Update one node (or many) in a Texinfo file with sequential pointers.
+
+This function causes the `Next' or `Previous' pointer to point to the
+immediately preceding or following node, even if it is at a higher or
+lower hierarchical level in the document.  Continually pressing `n' or
+`p' takes you straight through the file.
+
+Without any prefix argument, update the node in which point is located.
+Non-nil argument (prefix, if interactive) means update the nodes in the
+marked region.
+
+This command makes it awkward to navigate among sections and
+subsections; it should be used only for those documents that are meant
+to be read like a novel rather than a reference, and for which the
+Info `g*' command is inadequate."
+  t nil)
+
+(autoload 'texinfo-make-menu
+  "texnfo-upd"
+  "Without any prefix argument, make or update a menu.
+Make the menu for the section enclosing the node found following point.
+
+Non-nil argument (prefix, if interactive) means make or update menus
+for nodes within or part of the marked region.
+
+Whenever a menu exists, and is being updated, the descriptions that
+are associated with node names in the pre-existing menu are
+incorporated into the new menu.  Otherwise, the nodes' section titles
+are inserted as descriptions."
+  t nil)
 
 (autoload 'texinfo-all-menus-update
-          "texnfo-upd"
+  "texnfo-upd"
   "Update every regular menu in a Texinfo file.
 Remove pre-existing master menu, if there is one.
 
 If called with a non-nil argument, this function first updates all the
 nodes in the buffer before updating the menus."
-          t nil)
+  t nil)
 
 (autoload 'texinfo-master-menu
-          "texnfo-upd"
+  "texnfo-upd"
   "Make a master menu for a whole Texinfo file.
 Non-nil argument (prefix, if interactive) means first update all
 existing nodes and menus.  Remove pre-existing master menu, if there is one.
@@ -200,7 +142,7 @@ menus in the buffer (incorporating descriptions from pre-existing
 menus) before it constructs the master menu.
 
 The function removes the detailed part of an already existing master
-menu.  This action depends on the pre-exisitng master menu using the
+menu.  This action depends on the pre-existing master menu using the
 standard `texinfo-master-menu-header'.
 
 The master menu has the following format, which is adapted from the
@@ -218,55 +160,36 @@ recommendation in the Texinfo Manual:
 
 Each of the menus in the detailed node listing is introduced by the
 title of the section containing the menu."
-          t nil)
+  t nil)
 
 (autoload 'texinfo-indent-menu-description
-          "texnfo-upd"
+  "texnfo-upd"
   "Indent every description in menu following point to COLUMN.  
 Non-nil argument (prefix, if interactive) means indent every
 description in every menu in the region.  Does not indent second and
 subsequent lines of a multi-line description."
-          t nil)
-
-(autoload 'texinfo-sequential-node-update
-          "texnfo-upd"
-  "Update one node (or many) in a Texinfo file with sequential pointers.
-
-This function causes the `Next' or `Previous' pointer to point to the
-immediately preceding or following node, even if it is at a higher or
-lower hierarchical level in the document.  Continually pressing `n' or
-`p' takes you straight through the file.
-
-Without any prefix argument, update the node in which point is located.
-Non-nil argument (prefix, if interactive) means update the nodes in the
-marked region.
-
-This command makes it awkward to navigate among sections and
-subsections; it should be used only for those documents that are meant
-to be read like a novel rather than a reference, and for which the
-Info `g*' command is inadequate."
-          t nil)
+  t nil)
 
 (autoload 'texinfo-insert-node-lines
-          "texnfo-upd"
+  "texnfo-upd"
   "Insert missing `@node' lines in region of Texinfo file.
 Non-nil argument (prefix, if interactive) means also to insert the
 section titles as node names; and also to insert the section titles as
 node names in pre-existing @node lines that lack names."
-          t nil)
+  t nil)
 
 (autoload 'texinfo-start-menu-description
-          "texnfo-upd"
+  "texnfo-upd"
   "In this menu entry, insert the node's section title as a description. 
 Position point at beginning of description ready for editing.
 Do not insert a title if the line contains an existing description.
 
 You will need to edit the inserted text since a useful description
 complements the node name rather than repeats it as a title does."
-          t nil)
+  t nil)
 
 (autoload 'texinfo-multiple-files-update
-          "texnfo-upd"
+  "texnfo-upd"
   "Update first node pointers in each file included in OUTER-FILE;
 create or update main menu in the outer file that refers to such nodes. 
 This does not create or update menus or pointers within the included files.
@@ -300,9 +223,13 @@ Requirements:
 
 Thus, normally, each included file contains one, and only one,
 chapter."
-          t nil)
+  t nil)
 
 
+;;; Code:
+
+;;; Don't you dare insert any `require' calls at top level in this file--rms.
+
 ;;; Syntax table
 
 (defvar texinfo-mode-syntax-table nil)
@@ -320,6 +247,66 @@ chapter."
   (modify-syntax-entry ?} "){" texinfo-mode-syntax-table)
   (modify-syntax-entry ?\' "w" texinfo-mode-syntax-table))
 
+;; Written by Wolfgang Bangerth <zcg51122@rpool1.rus.uni-stuttgart.de>
+;; To override this example, set either `imenu-generic-expression'
+;; or `imenu-create-index-function'.
+(defvar texinfo-imenu-generic-expression
+  '((nil "^@node[ \t]+\\([^,\n]*\\)" 1)
+    ("Chapters" "^@chapter[ \t]+\\(.*\\)$" 1))
+
+  "Imenu generic expression for TexInfo mode.  See `imenu-generic-expression'.")
+
+(defvar texinfo-font-lock-keywords
+  '(;; All but the first 2 had an OVERRIDE of t.
+    ;; It didn't seem to be any better, and it's slower--simon.
+    ("^\\(@c\\|@comment\\)\\>.*" . font-lock-comment-face) ;comments
+    ;; Robert J. Chassell <bob@gnu.ai.mit.edu> says remove this line.
+                                        ;("\\$\\([^$]*\\)\\$" 1 font-lock-string-face t)
+    ("@\\([a-zA-Z]+\\|[^ \t\n]\\)" 1 font-lock-keyword-face) ;commands
+    ("^\\*\\(.*\\)[\t ]*$" 1 font-lock-function-name-face t) ;menu items
+    ("@\\(emph\\|strong\\|b\\|i\\){\\([^}]+\\)" 2 font-lock-comment-face)
+    ("@\\(file\\|kbd\\|key\\){\\([^}]+\\)" 2 font-lock-string-face)
+    ("@\\(samp\\|code\\|var\\|math\\){\\([^}]+\\)"
+     2 font-lock-variable-name-face)
+    ("@\\(cite\\|xref\\|pxref\\){\\([^}]+\\)" 2 font-lock-reference-face)
+    ("@\\(end\\|itemx?\\) +\\(.+\\)" 2 font-lock-function-name-face keep)
+    )
+  "Additional expressions to highlight in TeXinfo mode.")
+
+(defvar texinfo-section-list
+  '(("top" 1)
+    ("majorheading" 1)
+    ("chapter" 2)
+    ("unnumbered" 2)
+    ("appendix" 2)
+    ("chapheading" 2)
+    ("section" 3)
+    ("unnumberedsec" 3)
+    ("appendixsec" 3)
+    ("heading" 3)
+    ("subsection" 4)
+    ("unnumberedsubsec" 4)
+    ("appendixsubsec" 4)
+    ("subheading" 4)
+    ("subsubsection" 5)
+    ("unnumberedsubsubsec" 5)
+    ("appendixsubsubsec" 5)
+    ("subsubheading" 5))
+  "Alist of sectioning commands and their relative level.")
+
+(defun texinfo-outline-level ()
+  ;; Calculate level of current texinfo outline heading.
+  (save-excursion
+    (if (bobp)
+        0
+      (forward-char 1)
+      (let* ((word (buffer-substring-no-properties 
+                    (point) (progn (forward-word 1) (point))))
+             (entry (assoc word texinfo-section-list)))
+        (if entry
+            (nth 1 entry)
+          5)))))
+
 
 ;;; Keybindings
 (defvar texinfo-mode-map nil)
@@ -328,11 +315,11 @@ chapter."
 
 (defun texinfo-define-common-keys (keymap)
   "Define the keys both in Texinfo mode and in the texinfo-tex-shell."
-  (define-key keymap "\C-c\C-t\C-k"    'texinfo-kill-tex-job)
-  (define-key keymap "\C-c\C-t\C-x"    'texinfo-quit-tex-job)
-  (define-key keymap "\C-c\C-t\C-l"    'texinfo-recenter-tex-output-buffer)
-  (define-key keymap "\C-c\C-t\C-d"    'texinfo-delete-from-tex-print-queue)
-  (define-key keymap "\C-c\C-t\C-q"    'texinfo-show-tex-print-queue)
+  (define-key keymap "\C-c\C-t\C-k"    'tex-kill-job)
+  (define-key keymap "\C-c\C-t\C-x"    'texinfo-quit-job)
+  (define-key keymap "\C-c\C-t\C-l"    'tex-recenter-output-buffer)
+  (define-key keymap "\C-c\C-t\C-d"    'texinfo-delete-from-print-queue)
+  (define-key keymap "\C-c\C-t\C-q"    'tex-show-print-queue)
   (define-key keymap "\C-c\C-t\C-p"    'texinfo-tex-print)
   (define-key keymap "\C-c\C-t\C-i"    'texinfo-texindex)
 
@@ -349,10 +336,6 @@ chapter."
   ;; bindings for `texnfo-tex.el'
   (texinfo-define-common-keys texinfo-mode-map)
 
-  ;; bindings for `texinfmt.el'
-  (define-key texinfo-mode-map "\C-c\C-e\C-r"    'texinfo-format-region)
-  (define-key texinfo-mode-map "\C-c\C-e\C-b"    'texinfo-format-buffer)
-
   ;; bindings for `makeinfo.el'
   (define-key texinfo-mode-map "\C-c\C-m\C-k" 'kill-compilation)
   (define-key texinfo-mode-map "\C-c\C-m\C-l" 
@@ -360,6 +343,10 @@ chapter."
   (define-key texinfo-mode-map "\C-c\C-m\C-r" 'makeinfo-region)
   (define-key texinfo-mode-map "\C-c\C-m\C-b" 'makeinfo-buffer)
 
+  ;; bindings for `texinfmt.el'
+  (define-key texinfo-mode-map "\C-c\C-e\C-r"    'texinfo-format-region)
+  (define-key texinfo-mode-map "\C-c\C-e\C-b"    'texinfo-format-buffer)
+
   ;; bindings for updating nodes and menus
 
   (define-key texinfo-mode-map "\C-c\C-um"   'texinfo-master-menu)
@@ -371,9 +358,9 @@ chapter."
 
   (define-key texinfo-mode-map "\C-c\C-s"     'texinfo-show-structure)
 
-  (define-key texinfo-mode-map "\C-c\C-c}"    'up-list)
-  (define-key texinfo-mode-map "\C-c\C-c]"    'up-list) ; easier to type
-  (define-key texinfo-mode-map "\C-c\C-c{"    'texinfo-insert-braces)
+  (define-key texinfo-mode-map "\C-c}"          'up-list)
+  (define-key texinfo-mode-map "\C-c]"          'up-list)
+  (define-key texinfo-mode-map "\C-c{"          'texinfo-insert-braces)
 
   ;; bindings for inserting strings
 
@@ -397,8 +384,10 @@ chapter."
 
 (defvar texinfo-chapter-level-regexp 
   "chapter\\|unnumbered \\|appendix \\|majorheading\\|chapheading"
-  "*Regexp matching chapter-level headings (but not the top node).")
+  "Regular expression matching Texinfo chapter-level headings.
+This does not match `@node' and does not match the `@top' command.")
 
+;;;###autoload
 (defun texinfo-mode ()
   "Major mode for editing Texinfo files.
 
@@ -406,15 +395,14 @@ chapter."
 \\{texinfo-mode-map}
 
   These are files that are used as input for TeX to make printed manuals
-and also to be turned into Info files by \\[texinfo-format-buffer] or
-`makeinfo'.  These files must be written in a very restricted and
+and also to be turned into Info files with \\[makeinfo-buffer] or
+the `makeinfo' program.  These files must be written in a very restricted and
 modified version of TeX input format.
 
   Editing commands are like text-mode except that the syntax table is
 set up so expression commands skip Texinfo bracket groups.  To see
 what the Info version of a region of the Texinfo file will look like,
-use \\[texinfo-format-region].  This command runs Info on the current region
-of the Texinfo file and formats it properly.
+use \\[makeinfo-region], which runs `makeinfo' on the current region.
 
   You can show the structure of a Texinfo file with \\[texinfo-show-structure].
 This command shows the structure of a Texinfo file by listing the
@@ -494,54 +482,78 @@ value of texinfo-mode-hook."
   (setq comment-start-skip "@c +")
   (make-local-variable 'words-include-escapes)
   (setq words-include-escapes t)
-  (make-local-variable 'texinfo-start-of-header)
-  (setq texinfo-start-of-header "%**start")
-  (make-local-variable 'texinfo-end-of-header)
-  (setq texinfo-end-of-header "%**end")
+  (make-local-variable 'imenu-generic-expression)
+  (setq imenu-generic-expression texinfo-imenu-generic-expression)
+  (make-local-variable 'font-lock-defaults)
+  (setq font-lock-defaults '(texinfo-font-lock-keywords t))
+  (make-local-variable 'outline-regexp)
+  (setq outline-regexp 
+        (concat "@\\("
+                (mapconcat 'car texinfo-section-list "\\>\\|")
+                "\\>\\)"))
+  (make-local-variable 'outline-level)
+  (setq outline-level 'texinfo-outline-level)
+  (make-local-variable 'tex-start-of-header)
+  (setq tex-start-of-header "%**start")
+  (make-local-variable 'tex-end-of-header)
+  (setq tex-end-of-header "%**end")
   (run-hooks 'text-mode-hook 'texinfo-mode-hook))
 
-
-;;; Define start and end of header expressions 
-
-;; The start-of-header and end-of-header expressions are defined here
-;; since they used by both by `texinfmt.el' and by `texnfo-tex.el',
-;; only one of which is likely to be loaded at one time.
-
-(defvar texinfo-start-of-header "%**start"
-  "String to mark start of header for texinfo-tex-region.")
-
-(defvar texinfo-end-of-header "%**end"
-  "String to mark end of header for texinfo-tex-region.")
-
 
 ;;; Insert string commands
 
+;; Keep as concatinated lists for ease of maintenance
 (defconst texinfo-environment-regexp
-  "^@\\(f?table\\|enumerate\\|itemize\\|ifinfo\\|iftex\\|example\\|quotation\\|lisp\\|smallexample\\|smalllisp\\|display\\|format\\|flushleft\\|flushright\\|ignore\\|group\\|tex\\)"
+  (concat
+   "^@"
+   "\\("
+   "cartouche\\|"
+   "display\\|"
+   "end\\|"
+   "enumerate\\|"
+   "example\\|"
+   "f?table\\|"
+   "flushleft\\|"
+   "flushright\\|"
+   "format\\|"
+   "group\\|"
+   "ifhtml\\|"
+   "ifinfo\\|"
+   "iftex\\|"
+   "ignore\\|"
+   "itemize\\|"
+   "lisp\\|"
+   "macro\\|"
+   "multitable\\|"
+   "quotation\\|"
+   "smallexample\\|"
+   "smalllisp\\|"
+   "tex"
+   "\\)")
   "Regexp for environment-like TexInfo list commands.
-Subexpression 1 is what goes into the corresponding `@end' statement.")
+   Subexpression 1 is what goes into the corresponding `@end' statement.")
 
 ;; The following texinfo-insert-@end command not only inserts a SPC
 ;; after the @end, but tries to find out what belongs there.  It is
 ;; not very smart: it does not understand nested lists.
 
 (defun texinfo-insert-@end ()
-  "Insert the matching `@end' for a @table etc. in a texinfo buffer.
-BUG: Does not understand nested lists."
+  "Insert the matching `@end' for the last Texinfo command that needs one."
   (interactive)
-  (let ((string (save-excursion
-                (if (re-search-backward
-                     texinfo-environment-regexp nil t) 
-                    (buffer-substring (match-beginning 1)
-                                      (match-end 1))))))
+  (let ((depth 1) string)
+    (save-excursion
+      (while (and (> depth 0)
+                  (re-search-backward texinfo-environment-regexp nil t)
+                  (if (looking-at "@end")
+                      (setq depth (1+ depth))
+                    (setq depth (1- depth)))))
+      (looking-at texinfo-environment-regexp)
+      (if (zerop depth)
+          (setq string
+                (buffer-substring (match-beginning 1)
+                                  (match-end 1)))))
     (insert "@end ")
-    (if string (insert string))))
-
-; Earlier version of texinfo-insert-@end
-; (defun texinfo-insert-@end ()
-;   "Insert the string `@end ' (end followed by a space) in a texinfo buffer."
-;   (interactive)
-;   (insert "@end "))
+    (if string (insert string "\n"))))
 
 ;; The following insert commands accept a prefix arg N, which is the
 ;; number of words (actually s-exprs) that should be surrounded by
@@ -576,123 +588,87 @@ Use \\[up-list] to move forward out of the braces."
   (insert "{}")
   (backward-char))
 
-; (defun texinfo-insert-@code ()
-;   "Insert the string @code in a texinfo buffer."
-;   (interactive)
-;   (insert "@code{}")
-;   (backward-char))
-
 (defun texinfo-insert-@code (&optional arg)
-  "Insert the string @code in a texinfo buffer."
+  "Insert a `@code{...}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "code" arg))
 
-; (defun texinfo-insert-@dfn ()
-;   "Insert the string @dfn in a texinfo buffer."
-;   (interactive)
-;   (insert "@dfn{}")
-;   (backward-char))
-
 (defun texinfo-insert-@dfn (&optional arg)
-  "Insert the string @dfn in a texinfo buffer."
+  "Insert a `@dfn{...}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "dfn" arg))
 
 (defun texinfo-insert-@example ()
-  "Insert the string @example in a texinfo buffer."
+  "Insert the string `@example' in a Texinfo buffer."
   (interactive)
   (insert "@example\n"))
 
 (defun texinfo-insert-@file (&optional arg)
-  "Insert the string @file in a texinfo buffer."
+  "Insert a `@file{...}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "file" arg))
 
-; (defun texinfo-insert-@item ()
-;   "Insert the string @item in a texinfo buffer."
-;   (interactive)
-;   (insert "@item")
-;   (newline))
-
 (defun texinfo-insert-@item ()
-  "Insert the string @item in a texinfo buffer."
+  "Insert the string `@item' in a Texinfo buffer."
   (interactive)
   (insert "@item")
   (newline))
 
-; (defun texinfo-insert-@kbd ()
-;   "Insert the string @kbd in a texinfo buffer."
-;   (interactive)
-;   (insert "@kbd{}")
-;   (backward-char))
-
 (defun texinfo-insert-@kbd (&optional arg)
-  "Insert the string @kbd in a texinfo buffer."
+  "Insert a `@kbd{...}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "kbd" arg))
 
-; (defun texinfo-insert-@node ()
-;   "Insert the string @node in a texinfo buffer, 
-; along with a comment indicating the arguments to @node."
-;   (interactive)
-;   (insert "@node \n@comment  node-name,  next,  previous,  up")
-;   (forward-line -1)
-;   (forward-char 6))
-
 (defun texinfo-insert-@node ()
-  "Insert the string @node in a texinfo buffer, 
-along with a comment indicating the arguments to @node."
+  "Insert the string `@node' in a Texinfo buffer.
+This also inserts on the following line a comment indicating
+the order of arguments to @node."
   (interactive)
   (insert "@node \n@comment  node-name,  next,  previous,  up")
   (forward-line -1)
   (forward-char 6))
 
-; (defun texinfo-insert-@noindent ()
-;   "Insert the string @noindent in a texinfo buffer."
-;   (interactive)
-;   (insert "@noindent\n"))
-
 (defun texinfo-insert-@noindent ()
-  "Insert the string @noindent in a texinfo buffer."
+  "Insert the string `@noindent' in a Texinfo buffer."
   (interactive)
   (insert "@noindent\n"))
 
-; (defun texinfo-insert-@samp ()
-;   "Insert the string @samp in a texinfo buffer."
-;   (interactive)
-;   (insert "@samp{}")
-;   (backward-char))
-
 (defun texinfo-insert-@samp (&optional arg)
-  "Insert the string @samp in a texinfo buffer."
+  "Insert a `@samp{...}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "samp" arg))
 
 (defun texinfo-insert-@table (&optional arg)
-  "Insert the string @table in a texinfo buffer."
+  "Insert the string `@table' in a Texinfo buffer."
   (interactive "P")
   (insert "@table "))
 
-; (defun texinfo-insert-@var ()
-;   "Insert the string @var in a texinfo buffer."
-;   (interactive)
-;   (insert "@var{}")
-;   (backward-char))
-
 (defun texinfo-insert-@var (&optional arg)
-  "Insert the string @var in a texinfo buffer."
+  "Insert a `@var{}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "var" arg))
-
 
 ;;; Texinfo file structure
 
-; The following is defined in `texnfo-upd.el'
-; (defvar texinfo-section-types-regexp
-;   "^@\\(chapter \\|sect\\|subs\\|subh\\|unnum\\|major\\|chapheading \\|heading \\|appendix\\)"
-;   "Regexp matching chapter, section, other headings (but not the top node).")
+;; These are defined in texnfo-upd.el.
+;; texinfo-section-types-regexp
+;; texinfo-section-level-regexp
+;; texinfo-subsection-level-regexp
+;; texinfo-subsubsection-level-regexp
 
-; The other regexps are defined in `texnfo-upd.el' also.
+;; `texinfo-show-structure' requires texnfo-upd.el
 (defun texinfo-show-structure (&optional nodes-too) 
   "Show the structure of a Texinfo file.
 List the lines in the file that begin with the @-sign commands for
@@ -703,10 +679,9 @@ with @-sign commands for @chapter, @section, and the like, and list
 @node lines.
 
 Lines with structuring commands beginning in them are displayed in
-another window called the *Occur* window.  In that window, you can
-position the cursor over one of the lines and use
-\\[occur-mode-goto-occurrence], 
-to jump to the corresponding spot in the Texinfo file."
+another buffer named `*Occur*'.  In that buffer, you can move point to
+one of those lines and then use \\<occur-mode-map>\\[occur-mode-goto-occurrence], 
+to jump to the corresponding spot in the Texinfo source file."
 
   (interactive "P")
   (require 'texnfo-upd)
@@ -726,26 +701,30 @@ to jump to the corresponding spot in the Texinfo file."
       (re-search-forward ":")
       (setq margin
             (cond
-             ((looking-at (concat "@" texinfo-chapter-level-regexp)) 5)
+             ((looking-at
+               (concat "@\\(" texinfo-chapter-level-regexp "\\)")) 5)
              ;; ((looking-at "@chapter ") 5)
              ;; ((looking-at "@unnumbered ") 5)
              ;; ((looking-at "@appendix ") 5)
              ;; ((looking-at "@majorheading ") 5)
              ;; ((looking-at "@chapheading ") 5)
 
-             ((looking-at (concat "@" texinfo-section-level-regexp)) 9)
+             ((looking-at
+               (concat "@\\(" texinfo-section-level-regexp "\\)")) 9)
              ;; ((looking-at "@section ") 9)
              ;; ((looking-at "@unnumberedsec ") 9)
              ;; ((looking-at "@appendixsec ") 9)
              ;; ((looking-at "@heading ") 9)
 
-             ((looking-at (concat "@" texinfo-subsection-level-regexp)) 13)
+             ((looking-at 
+               (concat "@\\(" texinfo-subsection-level-regexp "\\)")) 13)
              ;; ((looking-at "@subsection ") 13)
              ;; ((looking-at "@unnumberedsubsec ") 13)
              ;; ((looking-at "@appendixsubsec ") 13)
              ;; ((looking-at "@subheading ") 13)
 
-             ((looking-at (concat "@" texinfo-subsubsection-level-regexp)) 17)
+             ((looking-at 
+               (concat "@\\(" texinfo-subsubsection-level-regexp "\\)")) 17)
              ;; ((looking-at "@subsubsection ") 17)
              ;; ((looking-at "@unnumberedsubsubsec ") 17)
              ;; ((looking-at "@appendixsubsubsec ") 17)
@@ -753,8 +732,201 @@ to jump to the corresponding spot in the Texinfo file."
              (t margin)))
       (indent-to-column margin)
       (beginning-of-line))))
-
 
-;;; Place provide at end of file.
+;;; The  tex  and  print  function definitions:
+
+(defvar texinfo-texi2dvi-command "texi2dvi"
+  "*Command used by `texinfo-tex-buffer' to run TeX and texindex on a buffer.")
+
+(defvar texinfo-tex-command "tex"
+  "*Command used by `texinfo-tex-region' to run TeX on a region.")
+
+(defvar texinfo-texindex-command "texindex"
+  "*Command used by `texinfo-texindex' to sort unsorted index files.")
+
+(defvar texinfo-delete-from-print-queue-command "lprm"
+  "*Command string used to delete a job from the line printer queue.
+Command is used by \\[texinfo-delete-from-print-queue] based on
+number provided by a previous \\[tex-show-print-queue]
+command.")
+
+(defvar texinfo-tex-trailer "@bye"
+  "String appended after a region sent to TeX by `texinfo-tex-region'.")
+
+(defun texinfo-tex-region (beg end)
+  "Run TeX on the current region.
+This works by writing a temporary file (`tex-zap-file') in the directory
+that is the value of `tex-directory', then running TeX on that file.
+
+The first line of the buffer is copied to the
+temporary file; and if the buffer has a header, it is written to the
+temporary file before the region itself.  The buffer's header is all lines
+between the strings defined by `tex-start-of-header' and `tex-end-of-header'
+inclusive.  The header must start in the first 100 lines.
+
+The value of `texinfo-tex-trailer' is appended to the temporary file after the region."
+  (interactive "r")
+  (require 'tex-mode)
+  (if (get-buffer "*tex-shell*")
+      (tex-kill-job)
+    (tex-start-shell))
+  (or tex-zap-file (setq tex-zap-file (make-temp-name "#tz")))
+  (let ((tex-out-file (concat tex-zap-file ".tex"))
+        (temp-buffer (get-buffer-create " tex-Output-Buffer"))
+        (zap-directory
+         (file-name-as-directory (expand-file-name tex-directory))))
+    (save-excursion
+      (save-restriction
+        (widen)
+        (goto-char (point-min))
+        (forward-line 100)
+        (let ((search-end (point))
+              (hbeg (point-min)) (hend (point-min))
+              (default-directory zap-directory))
+          (goto-char (point-min))
+          
+          ;; Copy first line, the `\input texinfo' line, to temp file
+          (write-region (point) 
+                        (save-excursion (end-of-line) (point))
+                        tex-out-file nil nil)
+          
+          ;; Don't copy first line twice if region includes it.
+          (forward-line 1)
+          (if (< beg (point)) (setq beg (point)))
+          
+          ;; Initialize the temp file with either the header or nothing
+          (if (search-forward tex-start-of-header search-end t)
+              (progn
+                (beginning-of-line)
+                (setq hbeg (point))     ; Mark beginning of header.
+                (if (search-forward tex-end-of-header nil t)
+                    (progn (beginning-of-line)
+                           (setq hend (point))) ; Mark end of header.
+                  (setq hbeg (point-min))))) ; Else no header.
+          
+          ;; Copy  header  to temp file.
+          (write-region (min hbeg beg) hend tex-out-file t nil)
+          
+          ;; Copy  region  to temp file.
+          (write-region (max beg hend) end tex-out-file t nil))
+        
+        ;; This is  a kludge to insert the tex-trailer into the tex-out-file.
+        ;;  We have to create a special buffer in which to insert
+        ;;  the tex-trailer first because there is no function with
+        ;;  which to append a literal string directly to a file.
+        (let ((local-tex-trailer texinfo-tex-trailer))
+          (set-buffer temp-buffer)
+          (erase-buffer)
+          ;; make sure trailer isn't hidden by a comment
+          (insert-string "\n")
+          (if local-tex-trailer (insert-string local-tex-trailer))
+          (tex-set-buffer-directory temp-buffer zap-directory)
+          (write-region (point-min) (point-max) tex-out-file t nil))
+
+;;; The following is sufficient in Emacs 19.
+;;;     (write-region (concat "\n" texinfo-tex-trailer) nil
+;;;                   tex-out-file t nil)
+        ))
+    
+    (tex-set-buffer-directory "*tex-shell*" zap-directory)
+    (tex-send-command tex-shell-cd-command zap-directory)
+    (tex-send-command texinfo-tex-command tex-out-file)
+    ;; alternatively:
+    ;;    (send-string "tex-shell" (concat tex-shell-cd-command " "
+    ;;                                     zap-directory "\n"))
+    ;;    (send-string "tex-shell" (concat texinfo-tex-command " "
+    ;;                                     tex-out-file "\n"))
+    (tex-recenter-output-buffer 0)))
+
+(defun texinfo-tex-buffer ()
+  "Run TeX on visited file, once or twice, to make a correct `.dvi' file."
+  (interactive)
+
+  ;; Make sure TeX shell is running.
+  (require 'tex-mode)
+  (if (get-buffer "*tex-shell*")
+      (quit-process (get-process "tex-shell") t)
+    (tex-start-shell))
+
+  (cond ((null buffer-file-name)
+         (error "Buffer not visiting any file!"))
+        ((buffer-modified-p)
+         (error "Buffer has been modified since last saved!")))
+
+  (setq tex-zap-file buffer-file-name)
+
+  (tex-send-command tex-shell-cd-command (file-name-directory tex-zap-file))
+  
+  (tex-send-command texinfo-texi2dvi-command tex-zap-file)
+
+  ;; alternatively:
+  ;;  (send-string "tex-shell"
+  ;;               (concat tex-shell-cd-command 
+  ;;                   " " (file-name-directory tex-zap-file) "\n"))
+  ;;  )
+  ;;
+  ;;  (send-string "tex-shell"
+  ;;           (concat texinfo-texi2dvi-command " " tex-zap-file "\n"))
+
+
+  (tex-recenter-output-buffer 0))
+
+(defun texinfo-texindex ()
+  "Run `texindex' on unsorted index files.
+The index files are made by \\[texinfo-tex-region] or \\[texinfo-tex-buffer].
+This runs the shell command defined by `texinfo-texindex-command'."
+  (interactive)
+  (require 'tex-mode)
+  (tex-send-command texinfo-texindex-command (concat tex-zap-file ".??"))
+  ;; alternatively
+  ;; (send-string "tex-shell"
+  ;;              (concat texinfo-texindex-command
+  ;;                      " " tex-zap-file ".??" "\n"))
+  (tex-recenter-output-buffer nil))
+
+(defun texinfo-tex-print ()
+  "Print `.dvi' file made by \\[texinfo-tex-region] or \\[texinfo-tex-buffer].
+This runs the shell command defined by `tex-dvi-print-command'."
+  (interactive)
+  (require 'tex-mode)
+  (tex-send-command tex-dvi-print-command (concat tex-zap-file ".dvi"))
+  ;; alternatively:
+  ;; (send-string "tex-shell"
+  ;;             (concat tex-dvi-print-command
+  ;;                     " " tex-zap-file ".dvi" "\n"))
+  (tex-recenter-output-buffer nil))
+
+(defun texinfo-quit-job ()
+  "Quit currently running TeX job, by sending an `x' to it."
+  (interactive)
+  (if (not (get-process "tex-shell"))
+      (error "No TeX shell running"))
+  (tex-send-command "x"))
+;; alternatively:
+;; save-excursion
+;;   (set-buffer (get-buffer "*tex-shell*"))
+;;   (goto-char (point-max))
+;;   (insert "x")
+;;   (comint-send-input)
+
+(defun texinfo-delete-from-print-queue (job-number)
+  "Delete job from the line printer spooling queue.
+You are prompted for the job number (use a number shown by a previous
+\\[tex-show-print-queue] command)."
+  (interactive "nPrinter job number for deletion: ")
+  (require 'tex-mode)
+  (if (tex-shell-running)
+      (tex-kill-job)
+    (tex-start-shell))
+  (tex-send-command texinfo-delete-from-print-queue-command job-number)
+  ;; alternatively
+  ;; (send-string "tex-shell"
+  ;;              (concat 
+  ;;               texinfo-delete-from-print-queue-command
+  ;;               " "
+  ;;               job-number"\n"))
+  (tex-recenter-output-buffer nil))
+
 (provide 'texinfo)
-;;;;;;;;;;;;;;;; end texinfo.el ;;;;;;;;;;;;;;;;
+
+;;; texinfo.el ends here
