@@ -1,13 +1,10 @@
-/* tilde.c -- Tilde expansion code (~/foo := $HOME/foo). */
+/* tilde.c -- Tilde expansion code (~/foo := $HOME/foo).
+   $Id: tilde.c,v 1.3 1996/09/29 23:12:30 karl Exp $
 
-/* This file is part of GNU Info, a program for reading online documentation
+   This file is part of GNU Info, a program for reading online documentation
    stored in Info format.
 
-   This file has appeared in prior works by the Free Software Foundation;
-   thus it carries copyright dates from 1988 through 1993.
-
-   Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993 Free Software
-   Foundation, Inc.
+   Copyright (C) 1988, 89, 90, 91, 92, 93, 96 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,7 +18,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written by Brian Fox (bfox@ai.mit.edu). */
 
@@ -37,21 +34,27 @@
 #  endif /* !AIX */
 #endif /* !__GNUC__ */
 
+#if defined (HAVE_STDLIB_H)
+#include <stdlib.h>
+#endif
+
 #include "tilde.h"
 #include <pwd.h>
 
-#if !defined (savestring)
-#define savestring(x) (char *)strcpy (xmalloc (1 + strlen (x)), (x))
-#endif /* !savestring */
+#if defined (HAVE_STRING_H)
+#include <string.h>
+#endif
+
+#include "clib.h"
 
 #if !defined (NULL)
 #  define NULL 0x0
 #endif
 
 #if defined (TEST) || defined (STATIC_MALLOC)
-static char *xmalloc (), *xrealloc ();
+static void *xmalloc (), *xrealloc ();
 #else
-extern char *xmalloc (), *xrealloc ();
+extern void *xmalloc (), *xrealloc ();
 #endif /* TEST || STATIC_MALLOC */
 
 /* The default value of tilde_additional_prefixes.  This is set to
@@ -70,7 +73,7 @@ static char *default_suffixes[] =
    standard meaning for expanding a tilde fails.  The function is called
    with the text (sans tilde, as in "foo"), and returns a malloc()'ed string
    which is the expansion, or a NULL pointer if there is no expansion. */
-Function *tilde_expansion_failure_hook = (Function *)NULL;
+CFunction *tilde_expansion_failure_hook = (CFunction *)NULL;
 
 /* When non-null, this is a NULL terminated array of strings which
    are duplicates for a tilde prefix.  Bash uses this to expand
@@ -211,7 +214,7 @@ tilde_expand_word (filename)
 {
   char *dirname;
 
-  dirname = filename ? savestring (filename) : (char *)NULL;
+  dirname = filename ? strdup (filename) : (char *)NULL;
 
   if (dirname && *dirname == '~')
     {
@@ -219,7 +222,8 @@ tilde_expand_word (filename)
       if (!dirname[1] || dirname[1] == '/')
 	{
 	  /* Prepend $HOME to the rest of the string. */
-	  char *temp_home = (char *)getenv ("HOME");
+	  extern char *getenv ();
+	  char *temp_home = getenv ("HOME");
 
 	  /* If there is no HOME variable, look up the directory in
 	     the password database. */
@@ -227,23 +231,24 @@ tilde_expand_word (filename)
 	    {
 	      struct passwd *entry;
 
-	      entry = getpwuid (getuid ());
+	      entry = (struct passwd *) getpwuid (getuid ());
 	      if (entry)
 		temp_home = entry->pw_dir;
 	    }
 
-	  temp_name = (char *)alloca (1 + strlen (&dirname[1])
-				      + (temp_home ? strlen (temp_home) : 0));
+	  temp_name = (char *)
+	    alloca (1 + strlen (&dirname[1])
+		    + (temp_home ? strlen (temp_home) : 0));
 	  temp_name[0] = '\0';
 	  if (temp_home)
 	    strcpy (temp_name, temp_home);
 	  strcat (temp_name, &dirname[1]);
 	  free (dirname);
-	  dirname = savestring (temp_name);
+	  dirname = strdup (temp_name);
 	}
       else
 	{
-	  struct passwd *getpwnam (), *user_entry;
+	  struct passwd *user_entry;
 	  char *username = (char *)alloca (257);
 	  int i, c;
 
@@ -256,7 +261,7 @@ tilde_expand_word (filename)
 	    }
 	  username[i - 1] = '\0';
 
-	  if (!(user_entry = getpwnam (username)))
+	  if (!(user_entry = (struct passwd *) getpwnam (username)))
 	    {
 	      /* If the calling program has a special syntax for
 		 expanding tildes, and we couldn't find a standard
@@ -265,13 +270,12 @@ tilde_expand_word (filename)
 		{
 		  char *expansion;
 
-		  expansion =
-		    (char *)(*tilde_expansion_failure_hook) (username);
+		  expansion = (*tilde_expansion_failure_hook) (username);
 
 		  if (expansion)
 		    {
-		      temp_name = (char *)alloca (1 + strlen (expansion)
-						  + strlen (&dirname[i]));
+		      temp_name = (char *)alloca
+			(1 + strlen (expansion) + strlen (&dirname[i]));
 		      strcpy (temp_name, expansion);
 		      strcat (temp_name, &dirname[i]);
 		      free (expansion);
@@ -282,13 +286,13 @@ tilde_expand_word (filename)
 	    }
 	  else
 	    {
-	      temp_name = (char *)alloca (1 + strlen (user_entry->pw_dir)
-					  + strlen (&dirname[i]));
+	      temp_name = (char *)alloca
+		(1 + strlen (user_entry->pw_dir) + strlen (&dirname[i]));
 	      strcpy (temp_name, user_entry->pw_dir);
 	      strcat (temp_name, &dirname[i]);
 	    return_name:
 	      free (dirname);
-	      dirname = savestring (temp_name);
+	      dirname = strdup (temp_name);
 	    }
 	    endpwent ();
 	}
@@ -333,23 +337,23 @@ main (argc, argv)
 
 static void memory_error_and_abort ();
 
-static char *
+static void *
 xmalloc (bytes)
      int bytes;
 {
-  char *temp = (char *)malloc (bytes);
+  void *temp = (void *)malloc (bytes);
 
   if (!temp)
     memory_error_and_abort ();
   return (temp);
 }
 
-static char *
+static void *
 xrealloc (pointer, bytes)
-     char *pointer;
+     void *pointer;
      int bytes;
 {
-  char *temp;
+  void *temp;
 
   if (!pointer)
     temp = (char *)malloc (bytes);
@@ -368,11 +372,5 @@ memory_error_and_abort ()
   fprintf (stderr, "readline: Out of virtual memory!\n");
   abort ();
 }
-
-/*
- * Local variables:
- * compile-command: "gcc -g -DTEST -o tilde tilde.c"
- * end:
- */
 #endif /* TEST */
 
