@@ -17,7 +17,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written by Brian Fox (bfox@ai.mit.edu). */
 
@@ -153,7 +153,7 @@ DECLARE_INFO_COMMAND (set_variable, "Set the value of an Info variable")
 	    REFERENCE *entry;
 
 	    entry = (REFERENCE *)xmalloc (sizeof (REFERENCE));
-	    entry->label = savestring (var->choices[i]);
+	    entry->label = strdup (var->choices[i]);
 	    entry->nodename = (char *)NULL;
 	    entry->filename = (char *)NULL;
 
@@ -260,7 +260,7 @@ make_variable_completions_array ()
       REFERENCE *entry;
 
       entry = (REFERENCE *)xmalloc (sizeof (REFERENCE));
-      entry->label = savestring (info_variables[i].name);
+      entry->label = strdup (info_variables[i].name);
       entry->nodename = (char *)NULL;
       entry->filename = (char *)NULL;
 
